import { isLowerCase, isTrim } from '#utils/ajv/method';
import { is } from 'drizzle-orm';

// 原子schema定义
export const username = {
    type: 'string',
    minLength: 4,
    maxLength: 32,
    pattern: '^[a-z][a-z0-9_]*$',
    isLowerCase: true,
    isTrim: true,
    description: '用户名（4-32位小写字母开头，允许数字和下划线）',
    errorMessage: {
        minLength: '用户名至少需要4个字符',
        maxLength: '用户名不能超过32个字符',
        pattern: '必须全部小写，且只能包含字母、数字和下划线，并以字母开头',
    },
};

export const email = {
    type: 'string',
    format: 'email',
    maxLength: 128,
    description: '用户电子邮箱地址',
    errorMessage: {
        format: '邮箱格式不正确',
        maxLength: '邮箱不能超过128个字符',
    },
};

export const password = {
    type: 'string',
    minLength: 8,
    maxLength: 128,
    pattern: '^(?=.*[a-z])(?=.*[A-Z])(?=.*\\d)(?=.*[@$!%*?&])[A-Za-z\\d@$!%*?&]+$',
    description: '密码需包含大小写字母、数字和特殊符号，8-128位',
    errorMessage: {
        minLength: '密码长度至少8位',
        maxLength: '密码不能超过128个字符',
        pattern: '必须包含至少一个小写字母、一个大写字母、一个数字和一个特殊字符',
    },
};
export const nickname = {
    type: 'string',
    minLength: 1,
    maxLength: 32,
    isTrim: true,
    description: '昵称',
    errorMessage: {
        minLength: '昵称至少需要1个字符',
        maxLength: '昵称不能超过32个字符',
    },
};

export const page = {
    type: 'integer',
    nullable: true,
    minimum: 1,
    default: 1,
    description: '页码（从1开始）',
    errorMessage: {
        type: '页码必须是整数',
        minimum: '页码最小为1'
    }
}

export const pageSize = {
    type: 'integer',
    nullable: true,
    minimum: 1,
    maximum: 100,
    default: 10,
    description: '每页条数',
    errorMessage: {
        type: '每页条数必须是整数',
        minimum: '每页条数范围1-100',
        maximum: '每页条数范围1-100'
    }
}
export const sortOrder = {
    type: 'string',
    nullable: true,
    enum: ['asc', 'desc'],
    default: 'desc',
    errorMessage: {
        type: '排序方式必须是字符串',
        enum: '排序方式只能是asc或desc'
    }
}
export const sortBy = {
    type: 'string',
    nullable: true,
    enum: ['createdAt', 'updatedAt', 'sort'],
    default: 'sort',
    description: '排序字段（默认createdAt）',
    errorMessage: {
        enum: '排序字段只能是createdAt/updatedAt/sort'
    }
}
export const sort = {
    type: 'integer',
    minimum: 0,
    maximum: 9999,
    default: 0,
    errorMessage: '排序值范围0-9999'
}
export const description = {
    type: ['string', 'null'],
    maxLength: 255,
    default: null,
    description: '描述信息'
}
export const pid = {
    type: 'integer',
    default: 0,
    description: '上级ID（0表示根节点）',
    errorMessage: {
        type: '上级ID必须是整数',
    }
}

export const module = {
    name: {
        type: 'string',
        minLength: 2,
        maxLength: 32,
        description: '模块名称（唯一标识）',
        errorMessage: {
            minLength: '模块名称至少需要2个字符',
            maxLength: '模块名称不能超过32个字符'
        }
    },
    moduleKey: {
        type: 'string',
        maxLength: 255,
        isLowerCase: true,
        isTrim: true,
        pattern: '^[a-z][a-z0-9_]*$',
        description: '模块唯一标识Key（英文小写）',
        errorMessage: {
            maxLength: '模块标识不能超过255个字符',
            pattern: '必须小写字母开头，只能包含字母、数字和下划线'
        }
    },
    status: {
        type: 'integer',
        enum: [0, 1, null],
        default: 0,
        errorMessage: {
            type: '状态值必须是整数',
            enum: '状态值只能是0(正常)或1(禁用)'
        }
    }
}

export const dict = {
    dictKey: {
        type:'string',
        maxLength: 255,
        isLowerCase: true,
        isTrim: true,
        pattern: '^[a-z][a-z0-9_]*$',
        description: '字典唯一标识Key（英文小写）',
        errorMessage: {
            maxLength: '模块标识不能超过255个字符',
            pattern: '必须小写字母开头，只能包含字母、数字和下划线'
        }
    },
    value: {
        type:'string',
        maxLength: 255,
        description: '字典值/名', 
    }
}

export const edit = {
    
    pid: {
        type: 'integer',
        description: '上级ID（0表示根节点）',
        errorMessage: {
            type: '上级ID必须是整数',
        },
        nullable: true
    },
    moduleId: {
        type: 'integer', examples: [0], default: 0, description: '模块',
        errorMessage: {
            type: '模块必须是整数',
        }
    },
    description: {
        type: ['string', 'null'],
        maxLength: 255,
        description: '描述信息', nullable: true
    },
    sort: {

        type: 'integer',
        minimum: 0,
        maximum: 9999,
        errorMessage: '排序值范围0-9999',
        nullable: true
    },
    status: {
        type: 'integer',
        enum: [0, 1, null],
        nullable: true,
        errorMessage: {
            type: '状态值必须是整数',
            enum: '状态值只能是0(正常)或1(禁用)'
        },
    }
}