import { sortOrder, module, sortBy, description, sort, pid, dict, edit } from "#schema/atomSchema";
import errorAtomSchema from "#schema/error.atomSchema";

const dictProperties = {
    id: { type: 'integer', examples: [1] },
    pid: { type: 'integer', examples: [0] },
    dictKey: { type: 'string', examples: ['gender'] },
    value: { type: 'string', examples: ['男'] },
    description: { type: ['string', 'null'] },
    sort: { type: 'integer', examples: [10] },
    status: { type: 'integer', enum: [0, 1] },
    moduleKey: { type: 'string', examples: [''] },
    moduleName: { type: 'string', examples: [''] },
    createdUser: { type: 'string', examples: [1001] },
    updatedUser: { type: 'string', examples: [1001] },
    createdBy: { type: 'string', examples: [1001] },
    updatedBy: { type: ['string', 'null'], examples: [1002] },
    createdAt: {
        type: 'string',
        format: 'date-time',
        examples: ['2023-07-15T08:23:45.000Z']
    },
    updatedAt: {
        type: 'string',
        format: 'date-time',
        examples: ['2023-07-16T09:34:12.000Z']
    }
}
// 通过PID获取字典列表
export const dictListSchema = {
    tags: ['系统管理'],
    summary: '获取字典列表',
    description: '通过PID获取字典列表（非分页）',
    querystring: {
        type: 'object',
        required: ['pid'],
        properties: {
            pid: {
                ...pid,
                nullable: true,
            },
            moduleKey: {
                type: 'string',
                maxLength: 255,
                default: null,
                description: '模块过滤',
                toNull: true,
                isLowerCase: true,
                isTrim: true,
                errorMessage: {
                    maxLength: '模块标识不能超过255个字符',
                },
            },
            sortBy: {
                ...sortBy,
                default: 'sort'
            },
            sortOrder
        }
    },
    response: {
        200: {
            type: 'object',
            properties: {
                code: { type: 'number', enum: [200] },
                data: {
                    type: 'array',
                    items: {
                        type: 'object',
                        properties: dictProperties
                    }
                }
            }
        },
        400: errorAtomSchema['400']
    },
    security: [{ apiKey: [] }]
};

// 字典详情
export const dictInfoSchema = {
    tags: ['系统管理'],
    summary: '获取字典详情',
    response: {
        200: {
            type: 'object',
            properties: {
                code: { type: 'number', enum: [200] },
                data: {
                    type: 'object',
                    properties: dictProperties
                },
                message: { type: 'string' }
            }
        }
    },
    security: [{ apiKey: [] }]
};

// 字典树schema 
export const dictTreeSchema = {
    tags: ['系统管理'],
    summary: '获取字典树',
    response: {
        200: {
            ...dictListSchema.response[200],
            properties: {
                code: { type: 'number', enum: [200] },
                data: {
                    type: 'object',
                    properties: {
                        ...dictProperties,
                        children: {
                            type: 'array',
                            items: { $ref: '#' }
                        }
                    }
                },
                message: { type: 'string' }
            }
        }
    },
    security: [{ apiKey: [] }]
};

// 添加字典
export const dictCreateSchema = {
    tags: ['系统管理'],
    summary: '创建字典项',
    body: {
        type: 'object',
        required: ['pid', 'moduleId', 'dictKey', 'value'],
        properties: {
            pid: {
                type: 'integer', examples: [0], default: 0, description: '上级字典ID（0表示根节点）',
                errorMessage: {
                    type: '上级ID必须是整数',
                }
            },
            moduleId: {
                type: 'integer', examples: [0], default: 0, description: '模块',
                errorMessage: {
                    type: '模块必须是整数',
                }
            },
            dictKey: { ...dict.dictKey },
            value: { ...dict.value },
            description,
            sort,
            status: { ...module.status }
        }
    },
    response: {
        201: {
            type: 'object',
            properties: {
                code: { type: 'number', enum: [201] },
                data: {
                    type: 'object',
                    properties: dictProperties
                },
                message: { type: 'string' }
            },
        },
        409: errorAtomSchema['409']
    },
    security: [{ apiKey: [] }]
};

// 修改字典
export const dictUpdateSchema = {
    tags: ['系统管理'],
    summary: '修改字典项',
    body: {
        type: 'object',
        properties: {
            dictKey: { ...dict.dictKey, nullable: true },
            value: { ...dict.value, nullable: true },
            moduleId: edit.moduleId,
            description: edit.description,
            sort: edit.sort,
            status: edit.status,
        },
        anyOf: [
            { required: ['dictKey'] },
            { required: ['value'] },
            { required: ['moduleId'] },
            { required: ['description'] },
            { required: ['sort'] },
            { required: ['status'] }
        ],
        additionalProperties: false
    },
    // response: dictCreateSchema.response,
    security: [{ apiKey: [] }]
};

// 删除字典
export const dictDeleteSchema = {
    tags: ['系统管理'],
    summary: '删除字典项',
    response: {
        204: {
            type: 'object',
            properties: {
                code: { type: 'number', enum: [204] },
                message: { type: 'string' }
            }
        },
        404: errorAtomSchema['404']
    },
    security: [{ apiKey: [] }]
};