import { email, username, password, nickname } from '#schema/atomSchema';
import errorAtomSchema from '#schema/error.atomSchema';
// 默认schema
export const userDefaultSchema = {};

// 新用户注册
export const userRegisterSchema = {
    tags: ['用户体系'],
    summary: '新用户注册',
    description: '创建新用户账号（需验证手机/邮箱）',
    body: {
        type: 'object',
        required: ['username', 'email', 'password', 'nickname'],
        errorMessage: {
            required: {
                username: '缺少用户名',
                email: '缺少邮箱',
                password: '缺少密码',
                nickname: '缺少昵称',
            },
        },
        properties: {
            username,
            email,
            password,
        },
    },
    response: {
        201: {
            type: 'object',
            properties: {
                code: { type: 'number' },
                data: { 
                    type: 'object',
                    properties: {
                        username: { type: 'string' },
                        nickname: { type: 'string' },
                        email: { type: 'string' },
                        userId: { 
                            type: 'string',
                            examples: ["692397256422330400"] 
                        },
                        pid: { 
                            type: 'string',
                            enum: [0] // 固定值
                        },
                        phone: { 
                            type: ['string', 'null'],
                            default: null
                        },
                        avatarUrl: { 
                            type: ['string', 'null'],
                            default: null
                        },
                        userType: { 
                            type: ['number', 'null'],
                            default: null
                        },
                        status: { 
                            type: 'number',
                            enum: [0, 1] // 假设 0-正常 1-禁用
                        },
                        createdBy: { 
                            type: 'number',
                            enum: [0] // 个人注册固定值
                        },
                        updatedBy: { 
                            type: ['number', 'null'],
                            default: null
                        },
                        createdAt: {
                            type: 'string',
                            format: 'date-time'
                        },
                        updatedAt: {
                            type: 'string',
                            format: 'date-time'
                        }
                    },
                    required: [
                        'username', 'email', 'userId', 'pid',
                        'status', 'createdBy', 'createdAt', 'updatedAt'
                    ],
                    additionalProperties: false // 严格模式
                },
                message: { type: 'string' }
            }
        },
        400: errorAtomSchema['400'],
        409: {
            description: '用户名/邮箱已存在',
            type: 'object',
            properties: {
                code: { type: 'number', enum: [409] },
                error: { type: 'string' },
            },
        },
    },
    security: [{}], // 不需要认证
};

// 用户登录
export const userLoginSchema = {
    tags: ['用户体系'],
    summary: '用户登录',
    description: '用户账号登录（用户名/邮箱+密码）',
    body: {
        type: 'object',
        required: ['password'],
        anyOf: [
            { required: ['username'] },
            { required: ['email'] }
        ],
        errorMessage: {
            required: {
                password: '缺少密码'
            },
            anyOf: '必须提供用户名或邮箱'
        },
        properties: {
            username,
            email,
            password
        },
    },
    response: {
        200: {
            description: '登录成功',
            type: 'object',
            properties: {
                code: { type: 'number', enum: [200] },
                data: { 
                    type: 'object',
                    properties: {
                        accessToken: { 
                            type: 'string',
                            examples: ["eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..."]
                        },
                        refreshToken: {
                            type: 'string',
                            examples: ["eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..."]
                        },
                        userInfo: {
                            type: 'object',
                            properties: {
                                userId: { type: 'string' },
                                username: { type: 'string' },
                                email: { type: 'string' },
                                avatarUrl: { type: ['string', 'null'] }
                            },
                            required: ['userId', 'username', 'email'],
                            additionalProperties: false
                        }
                    },
                    required: ['accessToken', 'refreshToken', 'userInfo'],
                    additionalProperties: false
                },
                message: { type: 'string' }
            }
        },
        400: errorAtomSchema['400'],
        401: errorAtomSchema['401']
    },
    security: [{}],
};

// 刷新token
export const userRefreshTokenSchema = {
    tags: ['用户体系'],
    summary: '刷新访问令牌',
    description: '使用refreshToken获取新的accessToken',
    body: {
        type: 'object',
        required: ['refreshToken'],
        properties: {
            refreshToken: {
                type: 'string',
                description: '使用refreshToken获取新的accessToken',
                examples: ["eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..."]
            }
        },
        errorMessage: {
            required: {
                refreshToken: '缺少刷新令牌'
            }
        },
    },
    response: {
        200: {
            description: '令牌刷新成功',
            type: 'object',
            properties: {
                code: { type: 'number', enum: [200] },
                data: {
                    type: 'object',
                    properties: {
                        accessToken: {
                            type: 'string',
                            examples: ["eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..."]
                        },
                        refreshToken: {
                            type: 'string',
                            examples: ["eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9..."]
                        }
                    },
                    required: ['accessToken', 'refreshToken'],
                    additionalProperties: false
                },
                message: { type: 'string' }
            }
        },
        400: {
            description: '无效的refreshToken',
            type: 'object',
            properties: {
                code: { type: 'number', enum: [400] },
                error: { type: 'string' },
            }
        },
        401: {
            description: '令牌已过期',
            type: 'object',
            properties: {
                code: { type: 'number', enum: [401] },
                error: { type: 'string' },
            }
        }
    },
    security: [{}] // 只需要携带refreshToken本身
};

