import { sysUser, sysUserAuth, sysUserAuthHistory, sysModule, sysDict } from '#entity';
import { or, eq, and, asc, desc, count, like, ne, sql } from 'drizzle-orm';
import { alias, QueryBuilder } from 'drizzle-orm/mysql-core';

// 获取字典列表
export async function getDictList(queryData) {
    const { pid = 0, moduleKey, sortBy = 'sort', sortOrder = 'asc' } = queryData;

    // 联表查询构造器
    const sysCreatedUser = alias(sysUser, 'sysCreatedUser');
    const sysUpdatedUser = alias(sysUser, 'sysUpdatedUser');
    const query = this.db
        .select({
            id: sysDict.id,
            pid: sysDict.pid,
            dictKey: sysDict.dictKey,
            value: sysDict.value,
            description: sysDict.description,
            sort: sysDict.sort,
            status: sysDict.status,
            moduleKey: sysModule.moduleKey, // 新增模块Key字段
            moduledName: sysModule.name, // 新增模块名称字段
            createdUser: sysCreatedUser.nickname,
            updatedUser: sysUpdatedUser.nickname,
            createdBy: sysModule.createdBy,
            updatedBy: sysModule.updatedBy,
            createdAt: sysModule.createdAt,
            updatedAt: sysModule.updatedAt
        })
        .from(sysDict)
        .leftJoin(sysModule, eq(sysDict.moduleId, sysModule.id)) // 通过moduleKey关联模块表
        .leftJoin(sysCreatedUser, eq(sysCreatedUser.userId, sysModule.createdBy))
        .leftJoin(sysUpdatedUser, eq(sysUpdatedUser.userId, sysModule.updatedBy))
        .$dynamic();

    // 构建过滤条件
    const conditions = [eq(sysDict.pid, pid === null ? 0 : pid)];
    if (moduleKey) {
        conditions.push(eq(sysModule.moduleKey, moduleKey)); // 根据模块Key过滤
    }

    // 应用排序规则
    const orderBy = sortOrder === 'desc'
        ? desc(sysDict[sortBy])
        : asc(sysDict[sortBy]);

    return await query
        .where(and(...conditions))
        .orderBy(orderBy);
}

// 新增字典详情查询方法
export async function getDictDetail(dictId) {
    const sysCreatedUser = alias(sysUser, 'sysCreatedUser');
    const sysUpdatedUser = alias(sysUser, 'sysUpdatedUser');

    const result = await this.db
        .select({
            id: sysDict.id,
            pid: sysDict.pid,
            dictKey: sysDict.dictKey,
            value: sysDict.value,
            description: sysDict.description,
            sort: sysDict.sort,
            status: sysDict.status,
            moduleKey: sysModule.moduleKey,
            moduleName: sysModule.name,
            createdUser: sysCreatedUser.nickname,
            updatedUser: sysUpdatedUser.nickname,
            createdAt: sysDict.createdAt,  // 修正为字典表字段
            updatedAt: sysDict.updatedAt    // 修正为字典表字段
        })
        .from(sysDict)
        .leftJoin(sysModule, eq(sysDict.moduleId, sysModule.id))
        .leftJoin(sysCreatedUser, eq(sysCreatedUser.userId, sysDict.createdBy))
        .leftJoin(sysUpdatedUser, eq(sysUpdatedUser.userId, sysDict.updatedBy))
        .where(eq(sysDict.id, dictId))
        .limit(1);

    return result[0] || null;
}

// 检查字典Key冲突
export async function checkDictKeyConflict(pid, dictKey) {
    const result = await this.db
        .select({ count: count() })
        .from(sysDict)
        .where(and(
            eq(sysDict.pid, pid),
            eq(sysDict.dictKey, dictKey)
        ));

    return result[0].count > 0;
}
// 检查字典Key冲突（排除当前记录）
export async function checkDictKeyConflictNotMe(pid, dictKey, excludeId) {
    const [result] = await this.db.select({ id: sysDict.id })
        .from(sysDict)
        .where(and(
            eq(sysDict.pid, pid),
            eq(sysDict.dictKey, dictKey),
            ne(sysDict.id, excludeId)
        ));
    return !!result;
}

// 通过pid和dictKey获取字典详情
export async function getDictByPidAndDictKey(pid, dictKey) {
    const sysCreatedUser = alias(sysUser, 'sysCreatedUser');
    const sysUpdatedUser = alias(sysUser, 'sysUpdatedUser');

    const result = await this.db
        .select({
            id: sysDict.id,
            pid: sysDict.pid,
            dictKey: sysDict.dictKey,
            value: sysDict.value,
            description: sysDict.description,
            sort: sysDict.sort,
            status: sysDict.status,
            moduleKey: sysModule.moduleKey,
            moduleName: sysModule.name,
            createdUser: sysCreatedUser.nickname,
            updatedUser: sysUpdatedUser.nickname,
            createdAt: sysDict.createdAt,  // 修正为字典表字段
            updatedAt: sysDict.updatedAt    // 修正为字典表字段
        })
        .from(sysDict)
        .leftJoin(sysModule, eq(sysDict.moduleId, sysModule.id))
        .leftJoin(sysCreatedUser, eq(sysCreatedUser.userId, sysDict.createdBy))
        .leftJoin(sysUpdatedUser, eq(sysUpdatedUser.userId, sysDict.updatedBy))
        .where(and(eq(sysDict.pid, pid), eq(sysDict.dictKey, dictKey)))
        .limit(1);

    return result[0] || null;
}

// 新增字典
export async function createDict(dictData) {
    await this.db.insert(sysDict).values({
        pid: dictData.pid,
        moduleId: dictData.moduleId, // 关联模块ID
        dictKey: dictData.dictKey,
        value: dictData.value,
        description: dictData.description,
        sort: dictData.sort || 0,
        status: dictData.status || 0,
        createdBy: dictData.createdBy,
    }).execute();
    return await getDictByPidAndDictKey.call(this, dictData.pid, dictData.dictKey);
}

// 更新字典数据
export async function updateDict(id, updateData) {
    await this.db.update(sysDict)
        .set({
            ...Object.entries(updateData).reduce((acc, [key, value]) => {
                if (value !== null && value !== undefined) {
                    acc[key] = value;
                }
                return acc;
            }, {}),
            updatedAt: new Date()
        })
        .where(and(
            eq(sysDict.id, id),
        ));

    return await getDictDetail.call(this, id);
}


// 删除字典项
export async function deleteDict(id) {
    const [result] = await this.db.delete(sysDict)
        .where(eq(sysDict.id, id))
        .execute();
    return result.affectedRows;
}

// 获取字典树
export async function getDictTree(dictId) {
    console.log(dictId);
    const sysCreatedUser = alias(sysUser, 'sysCreatedUser');
    const sysUpdatedUser = alias(sysUser, 'sysUpdatedUser');
    const dictList = {
        id: sysDict.id,
        pid: sysDict.pid,
        dictKey: sysDict.dictKey,
        value: sysDict.value,
        description: sysDict.description,
        sort: sysDict.sort,
        status: sysDict.status,
        moduleKey: sysModule.moduleKey,
        moduleName: sysModule.name,
        createdUser: sysCreatedUser.nickname,
        updatedUser: sysUpdatedUser.nickname,
        createdAt: sysDict.createdAt,  // 修正为字典表字段
        updatedAt: sysDict.updatedAt    // 修正为字典表字段
    }
    // ! 基础层级
    const baseQueryBuilder = new QueryBuilder();
    const baseQuery = baseQueryBuilder
        .select({
            ...dictList,
            level: sql`0`.as('level'),
        })
        .from(sysDict)
        .leftJoin(sysModule, eq(sysDict.moduleId, sysModule.id))
        .leftJoin(sysCreatedUser, eq(sysCreatedUser.userId, sysDict.createdBy))
        .leftJoin(sysUpdatedUser, eq(sysUpdatedUser.userId, sysDict.updatedBy))
        .where(eq(sysDict.id, dictId));

    // ! 递归层级
    const recursiveQueryBuilder = new QueryBuilder();
    const recursiveQuery = recursiveQueryBuilder
        .select({
            ...dictList,
            level: sql`dictHierarchy.level + 1`.as('level'),
        })
        .from(sysDict)
        .leftJoin(sysModule, eq(sysDict.moduleId, sysModule.id))
        .leftJoin(sysCreatedUser, eq(sysCreatedUser.userId, sysDict.createdBy))
        .leftJoin(sysUpdatedUser, eq(sysUpdatedUser.userId, sysDict.updatedBy))
        .innerJoin(sql`dictHierarchy`, sql`dictHierarchy.id = ${sysDict.pid}`);

    const rowName = customDrizzleRowWithRecursive(dictList);

    // ! 执行原始SQL查询
    return this.db.execute(
        sql`WITH RECURSIVE dictHierarchy(${rowName}) AS(${baseQuery} UNION ALL ${recursiveQuery}) SELECT * FROM dictHierarchy`,
    );
}

export function customDrizzleRowWithRecursive(obj) {
    // ! 获取所有的列别名
    const rowNameList = [...Object.keys(obj), 'level'];

    // ! 制造drizzle专属的列名称
    const rowName = sql.empty();
    rowNameList.forEach((i, index) => {
        rowName.append(sql`${sql.raw(i)}`);
        if (index < rowNameList.length - 1) {
            rowName.append(sql`, `);
        }
    });

    return rowName;
}