import { 
    getDictList,
    getDictDetail,
    checkDictKeyConflict,
    createDict,
    deleteDict,
    checkDictKeyConflictNotMe,
    updateDict,
    getDictTree
} from './dict.db.js';

// 字典列表服务
export async function dictListService(request) {
    const data = await getDictList.call(this, request.query);
    console.log('查询结果:', data); // 打印查询结果，用于调试和验证查询逻辑是否正确
    return { code: 200, data, message: '查询成功' };
}

// 字典详情服务
export async function dictInfoService(request) {
    const { id } = request.params;
    const data = await getDictDetail.call(this, id);
    
    if (!data) {
        return { code: 404, error: '字典不存在' };
    }
    
    return { 
        code: 200,
        data: data,
        message: '查询成功'
    };
}

// 字典树服务
export async function dictTreeService(request) {
    const data = await getDictTree.call(this, request.params.id);
    return { code: 200, data: data[0], message: '查询成功' };
}

// 创建字典服务（带分布式锁）
export async function dictCreateService(request, reply) {
    const { pid, moduleId, dictKey, value, description, sort, status } = request.body;
    const userId = request.user.userId; // 从JWT获取用户ID
    const lockKey = `${this.const.DISTRIBUTED_LOCK_PREFIX.CREATE_DICT}_${pid}_${dictKey}`;
    const lockIdentifier = this.snowflake();
    let renewInterval;

    try {
        // 获取分布式锁
        const locked = await this.redis.SET(lockKey, lockIdentifier, { NX: true, EX: 5 });
        if (!locked) throw this.httpErrors.tooManyRequests('操作正在进行，请稍后重试');

        // 启动锁续期
        renewInterval = setInterval(async () => {
            if (await this.redis.GET(lockKey) === lockIdentifier) {
                await this.redis.EXPIRE(lockKey, 5);
            }
        }, 3000);

        // 检查字典Key冲突
        const exists = await checkDictKeyConflict.call(this, pid, dictKey);
        if (exists) throw this.httpErrors.conflict('当前集合下字典标识已存在');

        // 创建新字典项
        const newDict = await createDict.call(this, { pid, moduleId, dictKey, value, description, sort, status, createdBy: userId }); // 确保 userId 正确传递给 createDict 方法;

        reply.code(201).send({
            code: 201,
            data: newDict,
            message: '字典创建成功'
        });
    } catch (err) {
        if (err.statusCode === 409) {
            throw this.httpErrors.conflict(err.message);
        }
        throw err;
    } finally {
        clearInterval(renewInterval);
        if (await this.redis.GET(lockKey) === lockIdentifier) {
            await this.redis.DEL(lockKey);
        }
    }
}


// 更新字典服务
export async function dictUpdateService(request) {
    const { id } = request.params;
    const updateData = request.body;
    const userId = request.user.userId;
    console.log('updateData:', updateData); // 打印更新数据，用于调试和验证更新逻辑是否正确
    
    let lockKey = `${this.const.DISTRIBUTED_LOCK_PREFIX.UPDATE_DICT}${id}`;
    // 检查字典是否存在
    const existing = await getDictDetail.call(this, id);
    if (!existing) throw this.httpErrors.notFound('字典不存在');
    // 如果更新了字典标识，需要改变锁key
    if (updateData.dictKey) {
        lockKey = `${this.const.DISTRIBUTED_LOCK_PREFIX.CREATE_DICT}_${existing.pid}_${updateData.dictKey}`;
    }
    
    const lockIdentifier = this.snowflake();
    let renewInterval;

    try {
        // 获取分布式锁
        const locked = await this.redis.SET(lockKey, lockIdentifier, { NX: true, EX: 5 });
        if (!locked) throw this.httpErrors.tooManyRequests('操作正在进行，请稍后重试');

        // 启动锁续期
        renewInterval = setInterval(async () => {
            if (await this.redis.GET(lockKey) === lockIdentifier) {
                await this.redis.EXPIRE(lockKey, 5);
            }
        }, 3000);

        // 检查字典Key冲突（如果更新了dictKey）
        if (updateData.dictKey && updateData.dictKey !== existing.dictKey) {
            const keyExists = await checkDictKeyConflictNotMe.call(
                this, 
                existing.pid,
                updateData.dictKey, 
                id
            );
            if (keyExists) throw this.httpErrors.conflict('字典标识已被占用');
        }

        // 执行更新
        const updated = await updateDict.call(this, id, {
            ...updateData,
            updatedBy: userId,
        });

        return {
            code: 200,
            data: updated,
            message: '字典更新成功'
        };
    } catch (err) {
        if (err.statusCode === 409) {
            throw this.httpErrors.conflict(err.message);
        }
        throw err;
    } finally {
        clearInterval(renewInterval);
        if (await this.redis.GET(lockKey) === lockIdentifier) {
            await this.redis.DEL(lockKey);
        }
    }
}

// 删除字典服务
export async function dictDeleteService(request) {
    const { id } = request.params;
    const data = await deleteDict.call(this, id);
    if(data === 0){
        return { code: 404, error: '字典不存在' };
    }else{
        return { code: 200, message: '删除成功' };
    }
}