import { sysUser, sysUserAuth, sysUserAuthHistory, sysModule } from '#entity';
import { or, eq, and, asc, desc, count, like, ne } from 'drizzle-orm';
import { alias } from 'drizzle-orm/mysql-core';

// 获取模块列表和分页
export async function getModuleList(queryData) {
    const {
        page,
        pageSize,
        name,
        moduleKey,
        status,
        sortBy = 'sort',
        sortOrder = 'desc'
    } = queryData;
    this.log.info(queryData);
    const sysCreatedUser = alias(sysUser, 'sysCreatedUser');
    const sysUpdatedUser = alias(sysUser, 'sysUpdatedUser');
    const isGetPage = page && pageSize;
    const listSelect = {
        id: sysModule.id,
        name: sysModule.name,
        moduleKey: sysModule.moduleKey,
        description: sysModule.description,
    }
    const pageSelect = {
        ...listSelect,
        version: sysModule.version,
        sort: sysModule.sort,
        status: sysModule.status,
        createdUser: sysCreatedUser.nickname,
        updatedUser: sysUpdatedUser.nickname,
        createdBy: sysModule.createdBy,
        updatedBy: sysModule.updatedBy,
        createdAt: sysModule.createdAt,
        updatedAt: sysModule.updatedAt
    }
    // 构建基础查询
    const db = this.db;
    let query = db
        .select(isGetPage ? pageSelect : listSelect)
        .from(sysModule)
        .leftJoin(sysCreatedUser, eq(sysCreatedUser.userId, sysModule.createdBy))
        .leftJoin(sysUpdatedUser, eq(sysUpdatedUser.userId, sysModule.updatedBy))
        .$dynamic();
    // 应用过滤条件
    const conditions = [];
    // 名称模糊查询
    if (name) conditions.push(like(sysModule.name, `%${name}%`));
    // 模块Key精确查询
    if (moduleKey) conditions.push(eq(sysModule.moduleKey, moduleKey));
    // 状态过滤
    if (status !== undefined) conditions.push(eq(sysModule.status, status));

    if (conditions.length > 0) {
        query.where(and(...conditions));
    }

    // 添加排序
    query = query.orderBy(
        sortOrder === 'asc'
            ? asc(sysModule[sortBy])
            : desc(sysModule[sortBy])
    );
    // 分页模式
    if (isGetPage) {
        const offset = (page - 1) * pageSize;
        // 分页数据
        const list = await query.limit(pageSize).offset(offset);
        // 总数查询
        const totalResult = await db
            .select({ count: count() })
            .from(sysModule)
            .where(and(...conditions));

        return {
            total: totalResult[0].count,
            list
        }
    } else {
        // 全量列表模式
        return {
            list: await query
        };
    }
}

// 查重模块名称和模块Key
export async function checkExistsNameAndKeyForModule(name, moduleKey) {
    const result = await this.db
        .select()
        .from(sysModule)
        .where(
            or(
                eq(sysModule.name, name), // 明确指定字段比较
                eq(sysModule.moduleKey, moduleKey),
            ),
        );

    return result.length > 0; // 正确判断结果集是否为空
}
// 新增模块插入方法
export async function insertModule(data) {
    return this.db.insert(sysModule).values({
        name: data.name,
        moduleKey: data.moduleKey,
        description: data.description,
        sort: data.sort || 0,
        status: data.status || 0,
        createdBy: data.createdBy,
    }).execute();
}

// 获取模块详情
export async function getModuleById(id) {
    const [result] = await this.db.select().from(sysModule).where(eq(sysModule.id, id)).limit(1);
    return result
}

// 检查模块Key冲突（排除自身）
export async function checkModuleKeyConflict(moduleKey, excludeId) {
    const [result] =  await this.db.select({ id: sysModule.id })
        .from(sysModule)
        .where(and(
            eq(sysModule.moduleKey, moduleKey),
            ne(sysModule.id, excludeId)
        ));
        return result
}

// 执行模块更新
export async function updateModule(id, updateData) {
    await this.db.update(sysModule)
        .set({
            // 动态过滤更新字段
            ...Object.entries(updateData).reduce((acc, [key, value]) => {
                // 只更新有实际值的字段（排除 null 和 undefined）
                if (value !== null && value !== undefined) {
                    acc[key] = value
                }
                return acc
            }, {}),
            updatedAt: new Date() // 确保更新时间总是被修改
        })
        .where(and(
            eq(sysModule.id, id),
            eq(sysModule.version, updateData.version)
        ));
        return await getModuleById.call(this, id)
}