import {
    getModuleList,
    checkExistsNameAndKeyForModule,
    insertModule,
    updateModule,
    getModuleById,
    checkModuleKeyConflict
} from "#services/module/module.db";
// 获取模块分页和列表
export async function moduleListService(request, reply) {
    const data = await getModuleList.call(this, request.query);
    return {
        code: 200,
        data,
        message: '获取成功'
    }
}

// 新增模块服务（带分布式锁）
export async function moduleCreateService(request, reply) {
    const { name, moduleKey } = request.body;
    const lockKey = `${this.const.DISTRIBUTED_LOCK_PREFIX.CREATE_MODULE}${moduleKey}`;
    const lockIdentifier = this.snowflake();
    let renewInterval;

    try {
        // 获取分布式锁
        const locked = await this.redis.SET(lockKey, lockIdentifier, { NX: true, EX: 5 });
        if (!locked) throw this.httpErrors.tooManyRequests('操作正在进行，请稍后重试');

        // 启动锁续期
        renewInterval = setInterval(async () => {
            if (await this.redis.GET(lockKey) === lockIdentifier) {
                await this.redis.EXPIRE(lockKey, 5);
            }
        }, 3000);

        // 检查模块Key是否存在
        const exists = await checkExistsNameAndKeyForModule.call(this, name, moduleKey,);
        if (exists) throw this.httpErrors.conflict('模块标识已存在');

        // 创建模块
        await insertModule.call(this, {
            ...request.body,
            createdBy: request.user?.userId || 0 // 从JWT获取用户ID
        });

        reply.code(201).send({
            code: 201,
            data: {
                moduleKey,
                name
            },
            message: '模块创建成功'
        });
    } catch (err) {
        if (err.statusCode === 409) {
            throw this.httpErrors.conflict('模块标识已存在');
        }
        throw err;
    } finally {
        clearInterval(renewInterval);
        if (await this.redis.GET(lockKey) === lockIdentifier) {
            await this.redis.DEL(lockKey);
        }
    }
}

// 模块修改服务
export async function moduleUpdateService(request, reply) {
    const { id } = request.params;
    const updateData = request.body;
    console.log(id, updateData);
    
    let lockKey = `${this.const.DISTRIBUTED_LOCK_PREFIX.UPDATE_MODULE}${id}`;
    if(updateData.moduleKey){
        lockKey = `${this.const.DISTRIBUTED_LOCK_PREFIX.CREATE_MODULE}${updateData.moduleKey}`;
    }
    const lockIdentifier = this.snowflake();
    let renewInterval;

    try {
        // 获取分布式锁
        const locked = await this.redis.SET(lockKey, lockIdentifier, { NX: true, EX: 5 });
        if (!locked) throw this.httpErrors.tooManyRequests('操作正在进行，请稍后重试');

        // 启动锁续期
        renewInterval = setInterval(async () => {
            if (await this.redis.GET(lockKey) === lockIdentifier) {
                await this.redis.EXPIRE(lockKey, 5);
            }
        }, 3000);

        // 检查模块是否存在
        const existing = await getModuleById.call(this, id);
        if (!existing) throw this.httpErrors.notFound('模块不存在');
        // 检查模块Key冲突（如果更新了moduleKey）
        if (updateData.moduleKey && updateData.moduleKey !== existing.moduleKey) {
            const keyExists = await checkModuleKeyConflict.call(this, updateData.moduleKey, id);
            
            if (keyExists) throw this.httpErrors.conflict('模块标识已被占用');
        }
        // 执行更新
        const updated = await updateModule.call(this, id, {
            ...updateData,
            updatedBy: request.user?.userId || 0,
            version: existing.version // 乐观锁
        });
        return {
            code: 200,
            data: updated,
            message: '模块更新成功'
        };
    } catch (err) {
        if (err.statusCode === 409) {
            throw this.httpErrors.conflict(err.message);
        }
        throw err;
    } finally {
        clearInterval(renewInterval);
        if (await this.redis.GET(lockKey) === lockIdentifier) {
            await this.redis.DEL(lockKey);
        }
    }
}