import { 
    checkExistsUsernameAndEmail,
    registerAccount,
    login,
    getUserInfoByUserId
 } from '#services/user/user.db';
// 新用户注册
export async function userRegisterService(request, reply) {
    // 1. 获取已验证的参数
    const { username, email, password } = request.body;

    // 2. 分布式锁
    // 生成分布式锁 key
    const lockKey = `${this.const.DISTRIBUTED_LOCK_PREFIX.REGISTER_USER}${username}:${email}`;
    // 生成唯一标识符（例如，使用Snowflake算法）
    const lockIdentifier = this.snowflake();
    // 生成新用户插入数据
    const newUserInsertData = {
        username,
        email,
        password: await this.hash(password),
        userId: lockIdentifier,
        pid: 0, // 个人注册账户父节点ID为0
        createdBy: 0, // 个人注册账户创建者ID为0
    }
    // 续锁定时器
    let renewInterval;

    try {
        // 3. 尝试获取分布式锁（NX: 仅当key不存在时设置，EX: 过期时间5秒）
        const locked = await this.redis.SET(lockKey, lockIdentifier, { NX: true, EX: 5 });
        if (!locked) throw this.httpErrors.tooManyRequests('操作正在进行，请稍后重试');

        // 4. 启动续锁定时器（每3秒续期一次）
        renewInterval = setInterval(async () => {
            const currentVal = await this.redis.GET(lockKey);
            if (currentVal === lockIdentifier) {
                await this.redis.EXPIRE(lockKey, 5); // 重置过期时间
            }
        }, 3000);
        // 5. 检测用户名和邮箱是否已存在
        const exists = await checkExistsUsernameAndEmail.call(this, username, email);
        if (exists) {
            throw this.httpErrors.conflict('用户名或邮箱已存在');
        }

        // 6. 注册用户
        const newUser = await registerAccount.call(this, newUserInsertData);
        reply
            .code(201)
            .send({
                code: 201,
                data: newUser,
                message: '用户创建成功'
            });
    } finally {
        if (renewInterval) clearInterval(renewInterval);
        // 释放锁逻辑（移除Lua脚本）
        if (await this.redis.GET(lockKey) === lockIdentifier) await this.redis.DEL(lockKey);
    }
}

// 用户登录
export async function userLoginService(request, reply) {
    // 1. 参数解析 ---------- 获取已验证参数（支持用户名/邮箱登录）
    const { username, email, password } = request.body;
    try {
        // 2. 用户查询 ---------- 查询用户信息
        const user = await login.call(this, username, email);
        console.log(0, user)
        // 3. 状态校验 ---------- 用户不存在或状态异常
        if (!user || user.status !== 0) {
            throw this.httpErrors.unauthorized('用户不存在或已被禁用');
        }
        
        // 4. 密码验证 ---------- 密码验证（使用注册时的哈希算法）
        const passwordMatch = await this.compareHash(password, user.password);
        if (!passwordMatch) {
            throw this.httpErrors.unauthorized('用户名或密码错误');
        }
        
        // 5. Token生成 ----------
        const accessToken = await this.jwt.sign(
            { userId: user.userId },
            { expiresIn: this.config.JWT_ACCESS_EXPIRES }
        );
        
        const refreshToken = await this.jwt.sign(
            { userId: user.userId },
            { expiresIn: this.config.JWT_REFRESH_EXPIRES }
        );
        
        // 6. 响应处理 ---------- 返回登录结果
        reply.code(200).send({
            code: 200,
            data: {
                accessToken,
                refreshToken,
                userInfo: {
                    userId: user.userId,
                    username: user.username,
                    email: user.email,
                    avatarUrl: user.avatarUrl
                }
            },
            message: '登录成功'
        });
    } catch (err) {
        // 错误处理 ---------- 统一处理认证错误
        if (err.statusCode === 401) throw err;
        this.log.error(`登录失败: ${err.message}`);
        throw this.httpErrors.unauthorized('认证失败');
    }
}

// 刷新Token
// ... existing code ...

export async function refreshTokenService(request, reply) {
    // 1. 参数解析 ---------- 获取已验证参数
    const { refreshToken } = request.body;

    try {
        // 2. Token验证 ---------- 验证refreshToken有效性
        const decoded = await this.jwt.verify(refreshToken);
        
        // 3. 用户查询 ---------- 查询用户状态
        const user = await getUserInfoByUserId.call(this, decoded.userId);
        
        // 4. 状态校验 ---------- 用户不存在或状态异常
        if (!user || user.status !== 0) {
            throw this.httpErrors.unauthorized('用户不存在或已被禁用');
        }

        // 5. 生成新Token ----------
        const newAccessToken = await this.jwt.sign(
            { userId: user.userId },
            { expiresIn: this.config.JWT_ACCESS_EXPIRES }
        );
        
        const newRefreshToken = await this.jwt.sign(
            { userId: user.userId },
            { expiresIn: this.config.JWT_REFRESH_EXPIRES }
        );

        // 6. 响应处理 ---------- 返回新Token
        reply.code(200).send({
            code: 200,
            data: {
                accessToken: newAccessToken,
                refreshToken: newRefreshToken
            },
            message: '令牌刷新成功'
        });
    } catch (err) {
        // 错误处理 ---------- 区分token验证错误类型
        if (err.name === 'JsonWebTokenError') {
            throw this.httpErrors.badRequest('无效的令牌格式');
        }
        if (err.name === 'TokenExpiredError') {
            throw this.httpErrors.unauthorized('令牌已过期');
        }
        this.log.error(`令牌刷新失败: ${err.message}`);
        throw err;
    }
}